--[[-----------------------------------------------------------------------------
Frame Container
-------------------------------------------------------------------------------]]
local Type, Version = "Frame", 999999
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local pairs, assert, type = pairs, assert, type
local wipe = table.wipe

-- WoW APIs
local PlaySound = PlaySound
local CreateFrame, UIParent = CreateFrame, UIParent

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: CLOSE

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Button_OnClick(frame)
	--PlaySound("gsTitleOptionExit")
	frame.obj:Hide()
end

local function Frame_OnClose(frame)
	frame.obj:Fire("OnClose")
end

local function Frame_OnMouseDown(frame)
	AceGUI:ClearFocus()
end

local function Title_OnMouseDown(frame)
	frame:GetParent():StartMoving()
	AceGUI:ClearFocus()
end

local function LayoutSizer_OnMouseUp(mover)
	local frame = mover:GetParent()
	frame:StopMovingOrSizing()
	local self = frame.obj
	local status = self.status or self.localstatus
	status.width = frame:GetWidth()
	status.height = frame:GetHeight()
	status.top = frame:GetTop()
	status.left = frame:GetLeft()
end

local function SizerSE_OnMouseDown(frame)
	frame:GetParent():StartSizing("BOTTOMRIGHT")
	AceGUI:ClearFocus()
end

local function SizerS_OnMouseDown(frame)
	frame:GetParent():StartSizing("BOTTOM")
	AceGUI:ClearFocus()
end

local function SizerE_OnMouseDown(frame)
	frame:GetParent():StartSizing("RIGHT")
	AceGUI:ClearFocus()
end

local function StatusBar_OnEnter(frame)
	frame.obj:Fire("OnEnterStatusBar")
end

local function StatusBar_OnLeave(frame)
	frame.obj:Fire("OnLeaveStatusBar")
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		self.frame:SetParent(UIParent)
		self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
		self:SetTitle()
		self:SetStatusText()
		self:ApplyStatus()
		self:Show()
        self:EnableResize(true)
	end,

	["OnRelease"] = function(self)
		self.status = nil
		wipe(self.localstatus)
	end,

	["OnWidthSet"] = function(self, width)
		local content = self.content
		local contentwidth = width - 34
		if contentwidth < 0 then
			contentwidth = 0
		end
		content:SetWidth(contentwidth)
		content.width = contentwidth
	end,

	["OnHeightSet"] = function(self, height)
		local content = self.content
		local contentheight = height - 57
		if contentheight < 0 then
			contentheight = 0
		end
		content:SetHeight(contentheight)
		content.height = contentheight
	end,

	["SetTitle"] = function(self, title)
		self.titletext:SetText(title)
		self.titlebg:SetWidth((self.titletext:GetWidth() or 0) + 10)
	end,

	["SetStatusText"] = function(self, text)
		self.statustext:SetText(text)
	end,

	["Hide"] = function(self)
		self.frame:Hide()
	end,

	["Show"] = function(self)
		self.frame:Show()
	end,

	["EnableResize"] = function(self, state)
		local func = state and "Show" or "Hide"
		self.sizer_se[func](self.sizer_se)
		self.sizer_s[func](self.sizer_s)
		self.sizer_e[func](self.sizer_e)
	end,

	-- called to set an external table to store status in
	["SetStatusTable"] = function(self, status)
		assert(type(status) == "table")
		self.status = status
		self:ApplyStatus()
	end,

	["ApplyStatus"] = function(self)
		local status = self.status or self.localstatus
		local frame = self.frame
		self:SetWidth(status.width or 700)
		self:SetHeight(status.height or 500)
		frame:ClearAllPoints()
		if status.top and status.left then
			frame:SetPoint("TOP", UIParent, "BOTTOM", 0, status.top)
			frame:SetPoint("LEFT", UIParent, "LEFT", status.left, 0)
		else
			frame:SetPoint("CENTER")
		end
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function SetHeadingStyle(frame)
	local playerColor = RAID_CLASS_COLORS[select(2, UnitClass('player'))]
	local InkedColors = {r = playerColor.r * 0.35, g = playerColor.g * 0.35,b = playerColor.b * 0.35,a = 0.9}
	
	local backdrop = CreateFrame("Frame", nil, frame,BackdropTemplateMixin and "BackdropTemplate")
	backdrop:SetPoint("TOP", frame, "TOP", 0, 12)
	backdrop:SetWidth(175)
	backdrop:SetHeight(18)

	local bg = backdrop:CreateTexture(nil, "BORDER")
	bg:SetAllPoints(backdrop)
	bg:SetColorTexture(InkedColors.r,InkedColors.g,InkedColors.b,InkedColors.a)

	local left = backdrop:CreateTexture(nil, "OVERLAY")
	left:SetColorTexture(0, 0, 0.95,0)
	left:SetPoint("TOPLEFT", 1, 0)
	left:SetPoint("BOTTOMLEFT", -1, 0)
	left:SetWidth(3)

	local right = backdrop:CreateTexture(nil, "OVERLAY")
	right:SetColorTexture(0, 0, 0.95, 0)
	right:SetPoint("TOPRIGHT", -1, 0)
	right:SetPoint("BOTTOMRIGHT", -1, 0)
	right:SetWidth(3)

	local bottom = backdrop:CreateTexture(nil, "OVERLAY")
	bottom:SetColorTexture(0, 0, 0.95, 0)
	bottom:SetPoint("BOTTOMLEFT", 1, 0)
	bottom:SetPoint("BOTTOMRIGHT", -1, 0)
	bottom:SetHeight(3)

	local top = backdrop:CreateTexture(nil, "OVERLAY")
	top:SetColorTexture(0, 0, 0.95, 0)
	top:SetPoint("TOPLEFT", 1, 3)
	top:SetPoint("TOPRIGHT", -1, 3)
	top:SetHeight(3)

	return backdrop
end

local function Constructor()
	local playerColor = RAID_CLASS_COLORS[select(2, UnitClass('player'))]
	local frame = CreateFrame("Frame", nil, UIParent,BackdropTemplateMixin and "BackdropTemplate")
	frame:Hide()

	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetResizable(true)
	frame:SetFrameStrata("FULLSCREEN_DIALOG")
	frame:SetMinResize(400, 200)
	frame:SetToplevel(true)
	frame:SetScript("OnHide", Frame_OnClose)
	frame:SetScript("OnMouseDown", Frame_OnMouseDown)
	local bg = frame:CreateTexture(nil, "BORDER")
	bg:SetAllPoints(frame)
	bg:SetColorTexture(playerColor.r*0.35,playerColor.g*0.35,playerColor.b*0.35, 0.9)
	

	local closebutton = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate",BackdropTemplateMixin and "BackdropTemplate")
	closebutton:SetScript("OnClick", Button_OnClick)
	closebutton:SetPoint("BOTTOMRIGHT", -27, 17)
	closebutton:SetHeight(20)
	closebutton:SetWidth(100)
	closebutton:SetText(CLOSE)
	
	closebutton.Left:Hide()
	closebutton.Right:Hide()
	closebutton.Middle:Hide()
	
	local tex = closebutton:CreateTexture(nil, "BACKGROUND")
	local playerColor = RAID_CLASS_COLORS[select(2, UnitClass('player'))]
	local InkedColors = {r = playerColor.r*0.85, g = playerColor.g*0.85,b = playerColor.b*0.85,a = 1}
	tex:SetAllPoints()
	tex:SetVertexColor(InkedColors.r, InkedColors.g, InkedColors.b, InkedColors.a)	
	tex:SetTexture("Interface\\AddOns\\Inked\\Graphics\\Buttons\\ButtonTextureUI")

	

	local statusbg = CreateFrame("Button", nil, frame,BackdropTemplateMixin and "BackdropTemplate")
	statusbg:SetPoint("BOTTOMLEFT", 15, 15)
	statusbg:SetPoint("BOTTOMRIGHT", -132, 15)
	statusbg:SetHeight(24)
	statusbg:SetScript("OnEnter", StatusBar_OnEnter)
	statusbg:SetScript("OnLeave", StatusBar_OnLeave)

	local statustext = statusbg:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	statustext:SetPoint("TOPLEFT", 7, -2)
	statustext:SetPoint("BOTTOMRIGHT", -7, 2)
	statustext:SetHeight(20)
	statustext:SetJustifyH("LEFT")
	statustext:SetText("")

	local titlebg = SetHeadingStyle(frame)

	local title = CreateFrame("Frame", nil, frame,BackdropTemplateMixin and "BackdropTemplate")
	title:EnableMouse(true)
	title:SetFrameLevel(titlebg:GetFrameLevel() + 20)
	title:SetScript("OnMouseDown", Title_OnMouseDown)
	title:SetScript("OnMouseUp", LayoutSizer_OnMouseUp)
	title:SetAllPoints(titlebg)

	local titletext = title:CreateFontString(nil, "OVERLAY", "Game36Font")
	titletext:SetPoint("TOP", titlebg, "TOP", 0, 0)
	titletext:SetFont("Interface\\AddOns\\Inked\\Fonts\\Prototype.ttf", 36,"THICKOUTLINE")

	local sizer_se = CreateFrame("Frame", nil, frame,BackdropTemplateMixin and "BackdropTemplate")
	sizer_se:SetPoint("BOTTOMRIGHT")
	sizer_se:SetWidth(25)
	sizer_se:SetHeight(25)
	sizer_se:EnableMouse()
	sizer_se:SetScript("OnMouseDown",SizerSE_OnMouseDown)
	sizer_se:SetScript("OnMouseUp", LayoutSizer_OnMouseUp)

	local sizer_s = CreateFrame("Frame", nil, frame,BackdropTemplateMixin and "BackdropTemplate")
	sizer_s:SetPoint("BOTTOMRIGHT", -25, 0)
	sizer_s:SetPoint("BOTTOMLEFT")
	sizer_s:SetHeight(25)
	sizer_s:EnableMouse(true)
	sizer_s:SetScript("OnMouseDown", SizerS_OnMouseDown)
	sizer_s:SetScript("OnMouseUp", LayoutSizer_OnMouseUp)

	local sizer_e = CreateFrame("Frame", nil, frame,BackdropTemplateMixin and "BackdropTemplate")
	sizer_e:SetPoint("BOTTOMRIGHT", 0, 25)
	sizer_e:SetPoint("TOPRIGHT")
	sizer_e:SetWidth(25)
	sizer_e:EnableMouse(true)
	sizer_e:SetScript("OnMouseDown", SizerE_OnMouseDown)
	sizer_e:SetScript("OnMouseUp", LayoutSizer_OnMouseUp)

	--Container Support
	local content = CreateFrame("Frame", nil, frame,BackdropTemplateMixin and "BackdropTemplate")
	content:SetPoint("TOPLEFT", 17, -27)
	content:SetPoint("BOTTOMRIGHT", -17, 40)

	local widget = {
		localstatus = {},
		titletext   = titletext,
		statustext  = statustext,
		titlebg     = titlebg,
		sizer_se    = sizer_se,
		sizer_s     = sizer_s,
		sizer_e     = sizer_e,
		content     = content,
		frame       = frame,
		type        = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end
	closebutton.obj, statusbg.obj = widget, widget

	return AceGUI:RegisterAsContainer(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
